'PropEKG Spin firmware Ver. 1d
'This code inputs both channels of EKG data from the PropEKG circuit board and transmits filtered data
'  over optically isolated serial link to PC for analysis, storage, and display.
'Additionally, onboard LEDs are flashed when a heartbeat is detected.
'Copyright 2008 Raymond Allen


CON
  'This section declares a 5 MHz crystal oscillator clock source for 80 MHz operation.
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  


  
VAR
    'variables for High and Low pass filters
    '(variables for both EKG channels delcared here although some are common to both)
    long dt    'time between samples
    long HPRC  'RC time constant for high-pass filter
    long LPRC  'RC time constant for low-pass filter 
    long HPalphaNumerator, HPalphaDenomenator      'pre-calculated numbers
    long LPalphaNumerator, LPalphaDenomenator   
    long HPynew[2], HPyold[2], HPxnew[2], HPxold[2]  'filter data
    long LPynew[2], LPyold[2], LPxnew[2] 


OBJ

    ser : "fullduplexserial"    'This object was provided by Parallax and implements the serial data link
    adc : "dual_adc1"           'This object digitizes analog EKG channel data using the sigma-delta technique




PUB main|i, acnt, ekg1, ekg2, StepCnt
  'This is the main routine

  'First, the serial link is activated by running this object in it's own cog
  '  mode 2 is required as the optoisolator inverts the transmitted signal
  '  using the programming pins 31 and 30 with a baud rate of 57600
  ser.start(31,30,2,57600) '(rxpin, txpin, mode, baudrate)

  'make LED pins outputs
  'Status LED is on Prop pin#7
  'Channel 1 heartbeat LED is on Prop pin#13
  dira[7]~~
  dira[13]~~
  dira[15]~~
                 
  'set guard output pins to ground
  'These pins are used to reduce noise on the pins used for Sigma-delta ADC
  'These are the pins on either side of the 2 pins actually used.
  dira[19]~~
  outa[19]~
  dira[21]~~
  outa[21]~
  dira[23]~~
  outa[23]~
  dira[8]~~
  outa[8]~
  dira[10]~~
  outa[10]~
  dira[12]~~
  outa[12]~
  
  'Start the sigma-delta object in it's own cog
  'Set for 12 bits for a sample rate of 19.5 KHz
  'Channel 1 on pins 22 and 20, Channel 2 on pins 9 and 11
  'Using the Averaging Window function of the ADC object to average over every 195 data points for an actual data rate of 100 Hz
  'The Peak Window function is not used 
        '(SampleBits, ipina, opina, nAverageWindowA, nPeakWindowA, ipinb, opinb, nAverageWindowB, nPeakWindowB
  adc.start(12,22,20, 195,195, 9,11, 195,195) 

  
  'Initialize clock and filter variables prior to main loop
  acnt:=cnt  'set inital clock count to current clock count
  InitFilterVariables  'initialize variables for the RC filters

  'Output data rate is based on EKG paper background:
  '   Each small square is 4 pixels wide.  There are 5 small squares per large square.  Five large squares is one second.
  '   Each pixel is then:  (200 ms / large square) * ( 1 large square / 20 pixels)= 10 ms
  '   Output period of 10 ms is a rate of 100 Hz

  'At 12 bits, ADC sample rate is 19500 Hz.
  '   With output rate of 100 Hz, there are 195 samples per data point.
  '   Setting ADC average window to 195 samples

  StepCnt:=ClkFreq/100  'Acquire data at 100 Hz 

  'Main Loop:  Infinite Loop
  repeat
    'wait until 80,000 clocks have passed
    if (cnt-acnt)>StepCnt  'Acquire data at 100 Hz
      acnt+=StepCnt  'reset inital clock count to current clock count  

      'Acquire, scale and filter input data
      ekg1:=Filter(-adc.GetAverageA*129/100,0)   'Filter and scale channel 1 data so that 160 levels = 0.5 mV
      ekg2:=Filter(-adc.GetAverageB*129/100,1)   'Filter and scale channel 2 data so that 160 levels = 0.5 mV

      'Transmit data as a continuous stream over optoisolated serial port
      'Sending in 3-byte packets:  (1= $F5, 2=Channel 1, 3=Channel 2)
      ser.tx($F5)
      ser.tx(ekg1>>3+128)  'Divide signal by 8 and shift to fit into a byte.  Now, 20 levels = 0.5 mV
      ser.tx(ekg2>>3+128)

      'Flash LEDs if channel1 > .3 mV to indicate heartbeat
      if ekg1>100  'this level may need to be adjusted for the best result
        outa[7]~~
        outa[13]~~ 
      else
        outa[7]~
        outa[13]~ 
      if ekg2>100  'this level may need to be adjusted for the best result
        outa[15]~~
        outa[7]~~ 
      else
        outa[15]~
        outa[7]~          


PRI InitFilterVariables| i
  'Initialize variables used for low and high pass filters
  'set time between data points
  dt:=    10000000/1000  'in tenths of microseconds (for 100Hz)
  'set RC time constant of High-Pass filter
  HPRC:=  10000000/10  '~1Hz
  'Set RC time constant of Low-Pass filter
  LPRC:=  10000000/20000  '~2kHz

  'pre-calculate numerators and denomenators to speed calculation
  HPalphaNumerator:=HPRC
  HPalphaDenomenator:=HPRC+dt
  LPalphaNumerator:=dt
  LPalphaDenomenator:=LPRC+dt

  'set initial conditions
  LPyold[0]:=0
  LPyold[1]:=0
  HPxold[0]:=0
  HPxold[1]:=0
        

PRI Filter(sample, channel)
  'Send data through simple numerical RC filters
  'Details on these algoriths is available here:
  '   http://en.wikipedia.org/wiki/High-pass_filter
  '   http://en.wikipedia.org/wiki/Low-pass_filter
  'The low-pass filter may not be required, but the high-pass filter probably is to keep the baseline constant.   

  'First, the high-pass filter
  HPxnew[channel]:= sample
  HPynew[channel]:=HPalphaNumerator*(HPyold[channel]+HPxnew[channel]-HPxold[channel])/HPalphaDenomenator
  HPxold[channel]:=HPxnew[channel]
  HPyold[channel]:=HPynew[channel]

  'Next, the low-pass filter
  LPxnew[channel]:=HPynew[channel]
  LPynew[channel]:= LPalphaNumerator*(LPxnew[channel]-LPyold[channel])/LPalphaDenomenator+LPyold[channel]
  LPyold[channel]:=LPynew[channel]

  'return the output of the low-pass filter
  return LPynew[channel]

    


DAT     'Not using this data section

   